(* : Title : L1Packv2.m--A Mathematica package for minimizing an $\ell_1$ penalized functional*)
(* : Author: Ignace Loris*)
(* : Summary: The L1Packv2 package implement a couple of algorithms for minimizing an L1 penalized functional: an exact algorithm, 
thresholded Landweber iteration, and others.*)
(* : Package Version: 2.0*)
(* : Keywords: optimization, minimization, 1-norm, least squares, inverse problem*)
(* : Warnings: not optimized for speed, thoroughly tested but may still contain bugs *)
(* : Limitations: < this is designed for real vectors/matrices >*)
(* : Discussion:  see documentation in pdf file, see remark *)
(* : Examples: see L1Packv2Demo.nb *)
(* : Remark: If you use this package in your work, please cite the technical report in your publications:
"L1Packv2: A Mathematica package for minimizing and l1-penalized functional", Ignace Loris (2008).
Bibliographic data in BibTeX format is easily downloadable from my website (list of publications):
http://homepages.vub.ac.be/~igloris *)
(* See also the list of references in the technical report for more information about the origin of the algorithms*)
(* free for non-profit use *)
      
BeginPackage["L1Packv2`"]

L1Packv2::usage = "L1Packv2.m implements the various algorithms for L1 penalized optimization."

SoftThreshold::usage= "SoftThreshold[x_,tau_]: Soft thresholding operation, NOT for complex numbers!"
ProjectionOnL1Ball::usage= "ProjectionOnL1Ball[x_List,R_,options___]: Projection of x on a L1 ball of radius R."
FindMinimizer::usage= "FindMinimizer[A_, y_, options___] finds minimizer of ||Ax-y||^2+2\[Tau]||x||_1 with exact algorithm."
CheckMinimizerList::usage="CheckMinimizerList[K_,y_,minlist_,lambdalist_,options___] checks whether minlist is a list of consecutive(!) nodes (for descending penalizations in lambdalist) in the minimizer corresponding to K and y, i.e. checks whether fixed point equation is satisfied between these nodes."
MinimizerInterpolationFunction::usage="MinimizerInterpolationFunction[minlist_,lambdalist_,var_] builds a list of interpolation functions based on the data provided, and evaluated in var."
ThresholdedLandweber::usage= "ThresholdedLandweber[mat_,data_,tau_,options___] implements the thresholded Landweber algorithm."
ProjectedLandweber::usage= "ProjectedLandweber[mat_,data_,radius_options___] implements the projected Landweber algorithm."
ProjectedSteepestDescent::usage= "ProjectedSteepestDescent[mat_,data_,radius_,options___]"
AdaptiveLandweber::usage= "AdaptiveLandweber[mat_,data_,radius_,numsteps_,options___]"
AdaptiveSteepestDescent::usage= "AdaptiveSteepestDescent[mat_,data_,radius_,numsteps_,options___]"
ConstrainedThresholdedLandweber::usage= "ConstrainedThresholdedLandweber[mat_,data_,A_,a_,tau_,options___] implements the thresholded Landweber algorithm."
StartingPoint::usage="An option used in some algorithms."
ListFunction::usage="An option used in some algorithms (variables: Minimizer, Counter, DataMisfit, Remainder, Time)."
MaximumNonZero::usage="An option used in FindMinimizer."
MaximumL1Norm::usage="An option used in FindMinimizer."
MinimumDiscrepancy::usage="An option used in FindMinimizer."
StoppingPenalty::usage="An option used in FindMinimizer."
Shift::usage="An option used in FindMinimizer."
Gram::usage="An option used in FindMinimizer."
StoppingCondition::usage="An option used in FindMinimizer etc."
Verbose::usage="An option used in some algorithms."
Exact::usage="An option used in some algorithms."
Weights::usage="An option used in some algorithms."
DataMisfit::usage="A variable that can be used in ListFunction."
Remainder::usage="A variable that can be used in ListFunction."
Time::usage="A variable that can be used in ListFunction."
Counter::usage="A variable that can be used in ListFunction."
Minimizer::usage="A variable that can be used in ListFunction."
Penalty::usage="A variable that can be used in ListFunction (only in FindMinimizer)."
Support::usage="A variable that can be used in ListFunction (only in FindMinimizer)."
Grammian::usage="A variable that can be used in ListFunction (only in FindMinimizer)."
FixedPointTest::usage="A variable that can be used in ListFunction (only in FindMinimizer)."
Tolerance::usage="Tolerance allowed in deciding the candidate indices for the next step. Default is zero."
CandidateIndex::usage="The index/indices that may be added in next step"

Begin["`Private`"]    (*begin the private/implementation part*)

(*Soft thresholding; assume that tau>0, Listable*) 
SoftThreshold[x_Real,tau_]:=If[Abs[x]<tau,0.,x-Sign[x]*tau]
   
SoftThreshold[x_,tau_]:=If[Abs[x]<tau,0,x-Sign[x]*tau]

ProjectionOnL1Ball[x_List,R_,options___]:=
  Module[{numberleft,restnodes,midpoint,taumid,l1valuemid,threshold,taua,taub,
      l1a,l1b,w2,weights},
      
    {weights}={Abs[Weights]}/.{options}/.{Weights->Table[1,{Length[x]}]};  
    w2=Sign[weights];  
      
    If[Norm[weights*x,1]<=R,x,
      restnodes=Union[Prepend[Sort[Abs[w2*x/(weights/.{0->1,0.->1})]],0]];
      numberleft=Length[restnodes];
      
      While[numberleft>2,
        midpoint=Round[numberleft/2];
        taumid=restnodes[[midpoint]];
        l1valuemid=Norm[weights*SoftThreshold[x,taumid*weights],1];
        If[l1valuemid==R,
          threshold=taumid;
          restnodes={taumid};,
          If[l1valuemid>R,
            restnodes=Take[restnodes,{midpoint,-1}];
            ,restnodes=Take[restnodes,midpoint];]
          ];
        numberleft=Length[restnodes];
        ];(*end while*)
        
      If[numberleft==1,threshold=taumid;,
        taua=restnodes[[1]];
        taub=restnodes[[2]];
        l1a=Norm[weights*SoftThreshold[x,taua*weights],1];
        l1b=Norm[weights*SoftThreshold[x,taub*weights],1];
        If[l1a==l1b,threshold=taua;,
          threshold=(R-l1a)*(taua-taub)/(l1a-l1b)+taua;];
        ];
      SoftThreshold[x,threshold*weights]
      ]
    ]



FindMinimizer[mat_,y_,options___]:=
  Module[{R,verbose,penalty,listfunction,lst,xnew,xold,signs,activeset,newindex,numdata,numvar,toolargecomp,
      KJ,GJ,newcol,temp,uJ,vJ,wJ,rex,wex,denom,minlist,pluslist,fullgammalist,
      gamma,largeuJ,iterate,flippedindices,zeroindices,gammalist,gammalist2,
      smallgammapositionlist,datalist,starttime,time,remainder,remainderold,remainder2,difference,differenceold,fixedpointtest,
      newindexpossibilities,nosolflag,trialnewindex,trialindices,trialnewindexcompl,startpt,
      badnewindices,badnewindices2,maxnonzero,weights,unweighted,zerosigns,
      stoppingcondition,stopcond,a1,a2,a3,discr1,discr2,discrep,alf,pen,penold,naturalstopcond,candidateindex,samesetstep,flippedcase,filteredgammalist,
      tolerance,almostzeroindices,alfaold,alfanew,shiftvec,Gmat,GmatJ,includeGmat},
    
    (* ========================== *)
    (*Switch off warnings: divide by zeroes can happen. Switch back on at the end*)
    (* ========================== *)
    Off[Power::"infy"];
    Off[Infinity::"indet"];
    
    
    (* ===================================== *)
    (* Initialisation                        *)
    (* ===================================== *)
    
    {numdata,numvar}=Dimensions[mat];
    
    (* handle options and set defaults *)
    {R,verbose,penalty,listfunction,maxnonzero,weights,stoppingcondition,discrep,tolerance,startpt,shiftvec,Gmat}=
    {MaximumL1Norm,Verbose,StoppingPenalty,HoldForm[ListFunction],MaximumNonZero,Weights,
    HoldForm[StoppingCondition],MinimumDiscrepancy,Tolerance,StartingPoint,Shift,Gram}/.{options}/.{MaximumL1Norm->Infinity,
    Verbose->1,StoppingPenalty->0,ListFunction->Null,MaximumNonZero->Infinity,
    Weights->Table[1,{numvar}],StoppingCondition->False,MinimumDiscrepancy->0,Tolerance->0,StartingPoint->Automatic,
    Shift->Table[0,{numvar}],Gram->{}};
    
    lst=listfunction/.{DataMisfit->difference, Remainder->remainder, FixedPointTest->fixedpointtest,
    Minimizer->xnew, Counter->iterate, Time->time, Penalty->pen, Grammian->GJ, Support->activeset,CandidateIndex->candidateindex};  
   
    
    stopcond=stoppingcondition/.{DataMisfit->difference, Remainder->remainder, Minimizer->xnew, 
    Counter->iterate, Time->time, Penalty->pen,Grammian->GJ,Support->activeset,CandidateIndex->candidateindex};
    
    (*check consistency of matrix size and data*)
    If[numdata!=Length[y],
    If[verbose>0,Print["Number of data not the same as number of rows in matrix. Breaking off..."];];
    Return[Indeterminate];
    ];
    
    (*check consistency of shift and number of vars*)
    If[numvar!=Length[shiftvec],
    If[verbose>0,Print["Number of variables not the same as size of Shift. Breaking off..."];];
    Return[Indeterminate];
    ];
    
    (*check consistency of Gmat and number of vars*)
    If[Gmat=={},includeGmat=False;,includeGmat=True];
    If[includeGmat && Dimensions[Gmat]!={numvar,numvar},
    If[verbose>0,Print["Size of Gram does not match number of variables. Breaking off..."];];
    Return[Indeterminate];
    ];
    
    (*check weights: positive and not all zero*)    
    If[MemberQ[weights,xx_/;xx<0],
    If[verbose>0,Print["All weights should be positive.\nNegative weights replaced by their absolute values."];];
    weights=Abs[weights];
    If[verbose>2,Print["weights: ",weights];];
    ];
    
    
    (*prepare to start algorithm*)  
    iterate=0;
    signs={};
    activeset={};
    maxindices={};
    zerosigns={};(*for unweighted components*)
    naturalstopcond=False;
    samesetstep=False;
    starttime=TimeUsed[];
    
      
    (*unweighted=unpenalized components*)  
    unweighted=Union[Flatten[Position[weights,0]],Flatten[Position[weights,0.]]];
    If[Length[unweighted]>0,
    If[verbose>1,Print[iterate,": There are unpenalized components: ",unweighted];];
    
    
    
    
    (* ========================================= *)
    (* Determine first step                      *)
    (* ========================================= *)
    
    If[Length[unweighted]==numvar,
    If[verbose>0,Print[iterate,": All components are unpenalized! This algorithm is not needed!"];];
    If[includeGmat,
    xnew=LinearSolve[Transpose[mat].mat+Gmat,Transpose[mat].y];,
    xnew=LinearSolve[Transpose[mat].mat,Transpose[mat].y];
    ];
    difference=y-mat.xnew;
    If[includeGmat,
    remainder=Transpose[mat].difference-shiftvec-Gmat.xnew;
    ,
    remainder=Transpose[mat].difference-shiftvec;
    ];
    pen=0;(*whatever*)
    If[verbose>1,
    fixedpointtest=Norm[remainder];
    Print[iterate,": test=",fixedpointtest];];
    time=TimeUsed[]-starttime;
    datalist={ReleaseHold[lst]};
    (*switch warnings back on*)
    On[Power::"infy"];
    On[Infinity::"indet"];
    If[datalist[[1]]===Null,Return[xnew],Return[{datalist,xnew}]];(*return minimizer*)
    ];
    ];
    
    
    
    (*=================================*)
    (*determine starting point*)
    If[startpt===Automatic,
    If[verbose>1,Print[iterate,": Finding starting point..."];];
    xold=Table[0,{numvar}];
    xnew=Table[0,{numvar}];
    (* if there are unweighted components, then we need to calculate the starting point *)
    If[Length[unweighted]>0,
    zerosigns=Table[0,{Length[unweighted]}];
    (*if unweighted components, starting point<>zero*)
    KJ=mat[[All,unweighted]];
    If[includeGmat,
    GmatJ=Gmat[[unweighted,unweighted]];
    temp=LinearSolve[Transpose[KJ].KJ+GmatJ,Transpose[KJ].y-shiftvec[[unweighted]]];
    ,
    temp=LinearSolve[Transpose[KJ].KJ,Transpose[KJ].y-shiftvec[[unweighted]]];
    ];
    
    If[Head[temp]===LinearSolve,
    If[verbose>0,Print["Cannot find starting point..."];];
    Return[Indeterminate];,
    xold[[unweighted]]=temp;];
    xnew=xold;
    ];(*end of IF length[unweighted]>0*)
    
    difference=y-mat.xold;
    If[includeGmat,
    remainder=Transpose[mat].difference-Gmat.xold-shiftvec;
    ,
    remainder=Transpose[mat].difference-shiftvec;
    ];      
    remainder2=remainder/weights;
    remainder2[[unweighted]]=0;
    pen=Max[Abs[remainder2]/.{Indeterminate->0,ComplexInfinity->0}];
    If[pen==0,
    candidateindex=Complement[Range[numvar],unweighted];,
    candidateindex=Complement[Flatten[Position[Abs[remainder]/weights,uu_/;((Abs[pen-uu]/pen)<=tolerance)]],unweighted];(*exclude unweighted, allow tolerance*)
    ];
    maxindices=candidateindex;
    ,
    (*starting point is given*)
    If[verbose>1,Print[iterate,": Starting point is given..."];];
    xold=startpt[[1]];
    xnew=xold;
    pen=startpt[[2]];
    activeset=Complement[startpt[[3]],unweighted];
    candidateindex=Complement[startpt[[4]],unweighted];
    maxindices=Union[activeset,candidateindex];
    difference=y-mat.xold;   
    If[includeGmat,
    remainder=Transpose[mat].difference-Gmat.xold-shiftvec;
    ,
    remainder=Transpose[mat].difference-shiftvec;
    ];
       
    remainder2=remainder/weights;
    remainder2[[unweighted]]=0;
    
    ];(*end if startingpoint*)
    
    
    (* print data for first step *)
    If[verbose>2,
    Print[iterate,": x= ",xold];
    Print[iterate,": r= ",remainder];
    Print[iterate,": r/w= ",remainder/weights];
    ];
    
    If[verbose>1,
    Print[iterate,": current penalty= ",pen];
    Print[iterate,": activeset= ",activeset];
    Print[iterate,": argmax|r/w|= ",maxindices];
    Print[iterate,": candidate new indices= ", candidateindex];
    ];
    
        
    (* check if x satisfies fixed point eqn *)
    (* ------------------------------------ *)
    fixedpointtest=Norm[xnew-SoftThreshold[xnew+remainder,pen*weights]];
    If[verbose>1,Print[iterate,": test= ",fixedpointtest]];
    
    (*append to data list*)
    time=TimeUsed[]-starttime;
    datalist={ReleaseHold[lst]};
    
    (* ======================================================== *)
    (* ======================================================== *)
    (* start main loop:                                         *)
    (* ======================================================== *)
    (* ======================================================== *)
    
    While[True,
      
      
      (* ==================================== *)
      (* Verify the various stopping criteria *)
      (* ==================================== *)
      
      
      (* stop if the algorithm reaches its natural end *)
      If[naturalstopcond,
        If[verbose>1,Print[iterate,": Algorithm has reached its final step. Stopping..."];];
        Break[];
        ];
      
      
      (* stop if l1-norm is higher than bound *)
      If[Norm[xnew*weights,1]>=R,
        If[verbose>1,Print[iterate,": l1-norm bound reached. Breaking off..."];];
        If[verbose>1,Print[iterate,": l1 norm of present step: ", Norm[xnew*weights,1],"; Target l1 norm: ", R];];
        Break[];
        ];
      
      
      
      (* stop if remainder is lower than bound *)
      If[pen<=penalty,
        If[verbose>1,
        Print[iterate,": Current penalty is smaller than lower bound. Breaking off..."];
        Print[iterate,": Penalty of present step: ",pen,"\[LessEqual]",penalty," ."];
        ];
        Break[];
        ];
      
      
      
      (* stop if number of nonzero components is higher than bound *)
      (* unweighted components are always counted as nonzero here, even if they happen to be zero by accident *)
       If[Length[activeset]+Length[unweighted]>=maxnonzero,
        If[verbose>1,
        Print[iterate,": Number of nonzero components has reached upper bound. Breaking off..."];
        Print[iterate,": Number of nonzero components of present step: ",Length[activeset],"+",
        Length[unweighted],">=",maxnonzero," ."];
        ];
        Break[];
        ];
      
      (* stop if discrepancy is lower than bound *)
      If[Norm[difference]^2<=discrep,
      If[verbose>1,
      Print[iterate,": Discrepancy has reached lower bound. Breaking off..."];
      Print[iterate,": Discrepancy of present step:",Norm[difference]^2,"<=",discrep," ."];
      ];
      Break[];
      ];
      
      
      (* stop if generic stopping condition is satified *)
      If[ReleaseHold[stopcond],
      If[verbose>1,
      Print[iterate,": Stopping Condition satisfied. Breaking off..."];
      Print[iterate,": ",Cases[{options},uu_Rule/;uu[[1]]==StoppingCondition][[1,2]]];
      ];
      Break[];
      ];
      
      (* stop if remainder is too large for an inactive component. This should never happen *)
      toolargecomp=Complement[Flatten[Position[Abs[remainder2],uu_/;uu>=pen,1]],candidateindex,activeset,unweighted];
      If[Length[toolargecomp]>0,
      If[verbose>0,
      Print[iterate,": Components ",toolargecomp," of remainder are too large (error or due to round-off). Breaking off."];];
      Break[];
      ];
      
      (* ==================================== *)
      (* ==================================== *)
      (* take next step                       *)
      (* ==================================== *)
      (* ==================================== *)
      
      iterate=iterate+1;
      
      (* ----------------------------------------------------------------------- *)
      (* First we need to determine the indices we need to add to the active set *)
      (* ----------------------------------------------------------------------- *)
      
      
      If[samesetstep && Length[candidateindex]>1,
      (* if there is a nonzero->zero and only 1 candidate index, then we MUST make a step with the same active set (without candidate index).*)
      (* Because we know that adding that candidate index will give a wrong direction. *)
      (* However , if there is more than 1 candidate index, we have to try all the possibilities to be sure*)
      If[verbose>1,Print[iterate,": There was a nonzero->zero component, but more than one candidate new index. Need to verify them all"];];
      samesetstep=False;
      (*if there is more than 1 candidate new index, the algorithm below that finds the correct subset, will also look at the empty set*)
      (* this means that it is still possible that the same acive set will be used*)
      (* we could do it this way in general, but we don't because it is exceptional and would slow us down.*)
      ];
      
      
      If[samesetstep,
      (* we come here if there was a nonzero->zero component in the previous step, and only one candiate index, which we know we have to disregard*)
      (* this means we have to handle the active set differently*)
      
      samesetstep=False;
      If[verbose>1,Print[iterate,": Have to handle case of same active set."];];
      (* if there is no OTHER zero (i.e. a second) component with maximum remainder it is ok because we already tested*)
      (* the case of all those indices being active (previous step). And the only possibility then is continuing with the same set*)
      activeset=Complement[activeset,zeroindices];
      newindex={activeset[[-1]]};
      activeset=Drop[activeset,-1];
      
      
      ,(*if not samesetstep*)
            
      If[Length[candidateindex]>1,
        (* ----------------------------------------------- *)
        (* special case of more than 1 candidate new index *)
        (* ----------------------------------------------- *)
        (* we need to choose which one(s) *)
        If[verbose>1,Print[iterate,": Warning: more than 1 candidate new index."];];
        
        
        
        (*===================================================*)
        (* start search for correct new index                *)
        (*===================================================*)
        
        
    
      If[verbose>1,Print[iterate,": Multiple candidate indices. Trying to find the right one(s)..."];];
    
      newindexpossibilities=Reverse[Subsets[candidateindex]];
      If[verbose>1,Print[iterate,": All possible choices of new index: ", newindexpossibilities];];
    
      Do[
      nosolflag=False;
      trialnewindex=newindexpossibilities[[i]];
      If[verbose>2,Print[iterate,": active set= ",activeset];];
      If[verbose>2,Print[iterate,": Trying the following new index: ",trialnewindex];];
      trialindices=Join[activeset,trialnewindex];
      KJ=mat[[All,Join[unweighted,trialindices]]];
      GJ=Transpose[KJ].KJ;
      
      If[includeGmat,
      GmatJ=Gmat[[Join[unweighted,trialindices],Join[unweighted,trialindices]]];
      GJ=Transpose[KJ].KJ+GmatJ;
      ,
      GJ=Transpose[KJ].KJ;
      ];
      
      uJ=LinearSolve[GJ,weights[[Join[unweighted,trialindices]]]*Join[zerosigns,Sign[remainder[[trialindices]]]]];
      
      (*check wether LinearSolve found a solution. If not dismiss the current choice*)
      
      If[Head[uJ]===LinearSolve,Continue[];];
      vJ=KJ.uJ;
      
      If[includeGmat,
      wJ=Transpose[mat].vJ+Gmat[[All,Join[unweighted,trialindices]]].uJ;,
      wJ=Transpose[mat].vJ;];
      
      
      largeuJ=Table[0,{numvar}];
      largeuJ[[Join[unweighted,trialindices]]]=uJ;
      
      If[verbose>2,Print[iterate,": r: ",remainder];];
      If[verbose>2,Print[iterate,": walking direction: ",largeuJ];];
      
      (*first check wether newindextrial components move in correct direction*)
      If[MemberQ[Sign[largeuJ[[trialnewindex]]*remainder2[[trialnewindex]]],-1],
        If[verbose>1,Print[iterate,": rejected because wrong direction..."];];
        Continue[];
        ];
      
      (* check that all newindextrial components of the direction are nonzero*)
      (* otherwise there will be a 0->0 and zero component will be included in the active set in the next step*)
      If[MemberQ[largeuJ[[trialnewindex]],0] || MemberQ[largeuJ[[trialnewindex]],0.],
        If[verbose>1,Print[iterate,": ",trialnewindex," rejected because no movement in one of the trial new indices..."];];
        Continue[];
        ];
      
      (*check that complement of newindextrial moves faster to zero than active set, i.e. >=1*)
      
      trialnewindexcompl=Complement[candidateindex,trialnewindex];
      
      If[verbose>1,Print[iterate,": newindex complement: ",trialnewindexcompl];];
      If[verbose>1,Print[iterate,": K*K walking direction: ",wJ];];
      
      (*the bad new indices are those for which the remainder would decrease more slowly
      than the active components, or faster but with the wrong sign*)
      
      badnewindices=
        Flatten[Position[Abs[wJ[[trialnewindexcompl]]]/weights[[trialnewindexcompl]],uu_/;uu<1]];
      badnewindices=trialnewindexcompl[[badnewindices]];
      (*check that the remaining ones (in trialnewindexcompl) have the right sign:*)  
       badnewindices2=Complement[trialnewindexcompl,badnewindices];
       badnewindices2=Select[badnewindices2,(Sign[wJ[[#]]]==Sign[-remainder2[[#]]])&];
       badnewindices=Union[badnewindices,badnewindices2]; 
        
      If[verbose>1,Print[iterate,": bad new indices: ",badnewindices];];
      
      (*stop searching if there are no badnewindices*)
      If[Length[badnewindices]==0,
      If[verbose>1,Print[iterate,": this choice is ok."];];
        Break[];];
        
      If[verbose>1,Print[iterate,": Continue because no good...(some components of remainder decrease to slowly or increase)"];];
      
      If[i==Length[newindexpossibilities],
        If[verbose>1,Print[iterate," Looked at all possibilities. No good solution found. Breaking off..."];];
        nosolflag=True;];
      
      (*if these two conditions are satisfied, then it must be ok...*)
      (*pass these indices to rest of algorithm*)
      
      ,{i,1,Length[newindexpossibilities]}];
    
      (* if no new index set found, then break*)
      If[nosolflag,Break[];];
      (* else update indices*)
      signs=Sign[remainder2[[activeset]]];
      newindex=trialnewindex;
      (*we need to reset KJ because we used that variable in the search*)
      KJ=mat[[All,Join[unweighted,activeset]]];
    
    
        
        (*===================================================*)
        (* end search for correct new index                  *)
        (*===================================================*)
        
        
        ,(*end of more than 1 candidate index, not samesetstep*)
        
        
        (*generic case of only 1 extra index in active set*)
        newindex=candidateindex;
        If[verbose>1,Print[iterate,": Just one new index candidate: ",newindex];];
        ];
      
      ];(*end If[samesetstep,,]*)
      
       
  
      
  
  
      (* Now we can take the following step *)
      (* ---------------------------------- *)  
      xold=xnew;
      differenceold=difference;
      remainderold=remainder;
      penold=pen;
      
      (* update sets*)
      activeset=Join[activeset,newindex];
      (* we need to pay special attention to the components that are not in the active set, but have maximum remainder*)
      specialindices=Complement[candidateindex,activeset];
      maxindices=Union[activeset,specialindices];
      If[verbose>1,
      Print[iterate, ": activeset= ",activeset];
      Print[iterate, ": special indices= ",specialindices];
      ];
      signs=Sign[remainder2[[activeset]]];
      
      KJ=mat[[All,Join[unweighted,activeset]]];
      If[includeGmat,
      GmatJ=Gmat[[Join[unweighted,activeset],Join[unweighted,activeset]]];
      GJ=Transpose[KJ].KJ+GmatJ;,
      GJ=Transpose[KJ].KJ;];
      
      If[verbose>3,
      Print[iterate, ": KJ= ",MatrixForm[KJ]];
      Print[iterate, ": GJ= ",MatrixForm[GJ]];];
      
      (*solve linear system*)
      uJ=LinearSolve[GJ,Join[zerosigns,signs]*Join[weights[[unweighted]],weights[[activeset]]]];
      (*check wether LinearSolve found a solution*)
      If[Head[uJ]===LinearSolve,
      If[verbose>0,Print[iterate,": Linear system has no solution. Breaking off..."];];
        Break[];
        ];

      vJ=KJ.uJ;
      If[includeGmat,
      wJ=Transpose[mat].vJ+Gmat[[All,Join[unweighted,activeset]]].uJ;
      ,wJ=Transpose[mat].vJ;];
      
      largeuJ=Table[0,{numvar}];
      largeuJ[[Join[unweighted,activeset]]]=uJ;
      
      If[verbose>2,
      Print[iterate,": x direction: ",largeuJ];
      Print[iterate,": r/w direction: ",wJ];];
      
      (* determine step length, first try *)
      (* -------------------------------- *)
      rex=remainder2[[activeset[[1]]]];
      wex=wJ[[activeset[[1]]]]/weights[[activeset[[1]]]];
      denom=wex-wJ/weights;
      denom[[activeset]]=1;
      minlist=(rex-remainder2)/denom/.ComplexInfinity->-888;(*/.ComplexInfinity->Infinity;*)
      denom=wex+wJ/weights;
      denom[[activeset]]=1;
      pluslist=(rex+remainder2)/denom/.ComplexInfinity->-888;(*/.ComplexInfinity->Infinity;*)
      minlist[[Join[unweighted,activeset]]]=-666;
      pluslist[[Join[unweighted,activeset]]]=-666;
      minlist=minlist/.DirectedInfinity[_]->-777;
      pluslist=pluslist/.DirectedInfinity[_]->-777;
      
      (*in case there are indices with maximum remainder, but not in active set, we need to be careful:*)
      (*for these we know some minlist or pluslist are exactly zero, this could be missed if inexact arithmetic:  *)
      (* they could also be 0/0, so we have to be careful later when determining the new candidateindex*)
      
      If[specialindices!={},
        Do[If[Sign[rex]==Sign[remainder2[[specialindices[[i]]]]],
            minlist[[specialindices[[i]]]]=-666,
            pluslist[[specialindices[[i]]]]=-666],{i,1,Length[specialindices]}];
      ];
      
      If[verbose>2,Print[iterate,": gamma lists: ",minlist,pluslist];];
             
      fullgammalist=Join[minlist,pluslist];
      
      (*set gamma zero in case not suitable gamma found, probably not necessary?*)
      gamma=0;
      
      filteredgammalist=DeleteCases[fullgammalist,dummy_/;dummy<=0];(*do not allow zero step length*)
      (*pick step length, but beware that later we will allow Tolerance when determining the corresponding candidate indices*)
      gamma=Min[filteredgammalist]; 
      If[verbose>1,Print[iterate,": gamma= ",gamma];];
      
      (*if all the components are in the active set (final stage), 
        then the previous rule for gamma does not apply. *)
      If[filteredgammalist=={},
        gamma=pen;(*this wil ensure remainder=0 or 0. at the end*)
        If[verbose>1,
        Print[iterate,": All components of remainder are equal (final(?) stage)."];
        Print[iterate,": special choice of gamma: ",gamma];];
        (* this situation is the last step in the algorithm, should force a stop, unless sign change, also stop if nonzero->zero*)
        naturalstopcond=True;(*this is not strictly necessary because pen will be 0. or 0 anyway, and the algorithm will stop*)
        (*see also later for checking gamma=pen leading to stop*)
        ];
        
        
        
      (* update x and candidateindex (provisionally) *)
      (* ------------------------------------------- *)  
      
      (*determine the candidate index from choice of gamma *)
      (* look only at level 1 of list, thus avoiding DirectedInfinity[number] matches...*)
      (* allow tolerance. gamma should never be zero, so it's ok to divide by gamma*)
      candidateindex=Complement[Union[
          Complement[Join[Flatten[Position[minlist,uu_/;((Abs[gamma-uu]/gamma)<=tolerance),1]],Flatten[Position[pluslist,uu_/;((Abs[gamma-uu]/gamma)<=tolerance),1]]],
            activeset]],unweighted];
      (*set naturalstopcond=true if filteredgammalist consists of copies of this gamma*)
      (*this will also be true if filteredgammalist is empty!*)
      If[Abs[gamma-pen]<=tolerance*Abs[pen],
      If[verbose>1,Print[iterate,": Gamma equals penalty. Final stage of first phase?"];];
      naturalstopcond=True;];     
       
      (* the above isn't always correct (complete). 0/0 could also be=gamma, so we have to check these too *)
      (* in other words, the previous specialindices must be verified again to see if they are still special*)
      If[specialindices!={},
        Do[
        If[Sign[rex]==Sign[remainder2[[specialindices[[i]]]]] && wex==(wJ/weights)[[specialindices[[i]]]],
        candidateindex=Union[candidateindex,{specialindices[[i]]}];
        If[verbose>1,Print[iterate,": Special index still special: ", specialindices[[i]]];];
        ];
        
        If[Sign[rex]==-Sign[remainder2[[specialindices[[i]]]]] && wex==-(wJ/weights)[[specialindices[[i]]]],
        candidateindex=Union[candidateindex,{specialindices[[i]]}];
        If[verbose>1,Print[iterate,": Special index still special: ", specialindices[[i]]];];
        ];
        
        ,{i,1,Length[specialindices]}];
      ];(*end special indices*)
      
      
      maxindices=Union[maxindices,candidateindex];
      If[verbose>1,Print[iterate,": corresponding candidate index: ", candidateindex];];  
             
      xnew=xold+gamma*largeuJ;
      If[verbose>2,Print[iterate,": new x without any corrections= ", xnew];];
      (*we will change nonzero->almost zero to nonzero->zero in active set*)
      (* thereby avoiding insignificant sign changes wrongly leading to cancelation of naturalstopcond*)
      almostzeroindices=Flatten[Position[Transpose[Abs[{xold[[activeset]],xnew[[activeset]]}]],
      uu_List/;(uu[[2]]<=tolerance*uu[[1]]),1]];
      almostzeroindices=activeset[[almostzeroindices]];
      If[almostzeroindices!={},
      xnew[[almostzeroindices]]=0;
      If[verbose>1,Print[iterate,": nonzero->almost zero changed into nonzero->zero: ",almostzeroindices];];
      ];
          
      
      (* check wether signs have flipped. If so, take a smaller step *)
      (* ----------------------------------------------------------- *)
      (*we compare with sign of (old) x. old remainder could be zero *)
      (* position in level 1 of list*)
      flippedindices=Complement[Flatten[Position[Sign[xold*xnew],-1,1]],unweighted];
      flippedcase=False;
      If[flippedindices!={},
      (* if there is a sign flip, we are not yet in final stage*)
      naturalstopcond=False;
      flippedcase=True;
      
      If[verbose>1,Print[iterate,": Sign change in active components: ", flippedindices, ". Correcting."];];
      gammalist=-xold[[flippedindices]]/largeuJ[[flippedindices]];
      If[verbose>2,Print[iterate,": new gamma list (flipped indices): ", gammalist];];
      gammalist2=DeleteCases[gammalist,dummy_/;dummy<=0];
      gamma=Min[DeleteCases[gammalist2,Indeterminate]];
      If[verbose>1,Print[iterate,": new (smaller) gamma: ", gamma];];
      (*determine the newindex from choice of gamma *)
      smallgammapositionlist=Flatten[Position[gammalist,gamma,1]]; (*position in level 1 of list*)
      candidateindex=Union[Complement[flippedindices[[smallgammapositionlist]],unweighted]];
      (* update x *)
      xnew=xold+gamma*largeuJ;
      (* need to set the ones that are set to zero, explicitly zero, due to possible round-off error*)
      xnew[[candidateindex]]=0;
      If[verbose>2,Print[iterate,": xnew corrected for sign flips: ", xnew];];
      (* need to update the active set itself also, and the matrix KJ*)    
      (* this will be handled in the nonzero->zero case*)
      (* also candidateindex is not always correct. maybe some specialindices  survived*)
      (* anyway: if there is a sign flip the index sets are always determined by the nonzero->zero case *)
      ];(*end of flipped indices case, will be handed over to nonzero->zero case...*)
      
      
      
      (* check wether components are put to zero       *)
      (* --------------------------------------------- *)
      (* this certainly happens when there was a sign flip *)
      (* position in level 1 of lists*)
      zeroindices=Complement[Union[Flatten[Position[xnew,0,1]],Flatten[Position[xnew,0.,1]]],
      Union[Flatten[Position[xold,0,1]],Flatten[Position[xold,0.,1]]]];
      zeroindices=Complement[zeroindices,unweighted];
      If[zeroindices!={},
        If[verbose>1,
        Print[iterate,": Nonzero-to-zero indices: ",zeroindices];
        Print[iterate,": Will try an extra step with same support at next step."];
        ];        
                
        activeset=Complement[activeset,zeroindices];
        If[flippedcase,
        (*is a sign flip occured, gamma is strictly smaller than initial guess, and we have to disregard the first guess of candidateindex*)
        candidateindex=zeroindices;
        ,
        (*if no sign flips, but only nonzero->zero exactly, we have to take into account the candiateindex found above!*)
        If[verbose>1,Print[iterate,": No sign flips, only nonzero->zero components. Adding these to the candidate indices..."];];
        candidateindex=Union[zeroindices,candidateindex];
        ];
        
        (* sometimes, we need to add specialindices that have remained special!*)
        If[specialindices!={},
        Do[
        If[Sign[rex]==Sign[remainder2[[specialindices[[i]]]]] && wex==(wJ/weights)[[specialindices[[i]]]],
        candidateindex=Union[candidateindex,{specialindices[[i]]}];
        If[verbose>1,Print[iterate,": Special index added to candidate index: ", specialindices[[i]]];];
        ];
        
        If[Sign[rex]==-Sign[remainder2[[specialindices[[i]]]]] && wex==-(wJ/weights)[[specialindices[[i]]]],
        candidateindex=Union[candidateindex,{specialindices[[i]]}];
        If[verbose>1,Print[iterate,": Special index added to candidate index: ", specialindices[[i]]];];
        ];
        
        ,{i,1,Length[specialindices]}];
      ];(*end special indices*)
        
        
        
        If[verbose>1,Print[iterate,": candidate new indices: ",candidateindex];];
        maxindices=Union[activeset,candidateindex];
        
        (* we need to do the step with same support. it is not like a generic step*)
        (* in a generic step, a zero->nonzero component is added. Here the components are already nonzero*)
        (* so we set a flag and handle this special case at the start of the next step*)
        
        samesetstep=True;
          
          ];(*end of nonzero->zero case*)
      
      
      (* update maxindices, could be wrong otherwise in step after nonzero->zero case *)
      (* ---------------------------------------------------------------------------- *)
      
      maxindices=Union[activeset,candidateindex];      
            
            
      (* calculate remainder etc.  *)
      (* ------------------------- *)      
            
            
      If[verbose>1,
        Print[iterate,": At the end of this step:"];
        Print[iterate,": activeset= ",activeset];
        Print[iterate,": argmax|r/w|= ",maxindices];
        Print[iterate,": candidate new indices= ", candidateindex];
        ];
           
      If[verbose>2,
        Print[iterate,": x= ",xnew];
        ];     
           
            
      
      difference=difference-gamma*vJ;
      remainder=remainder-gamma*wJ;
      remainder2=remainder/weights;
      If[verbose>2,
      Print[iterate,": r  = ",remainder];
      Print[iterate,": r/w= ",remainder2];];
      
      (*because gamma=pen in final stage, the following guarantees pen=0 or 0. at the end, even though other components may equal epsilon*)
      pen=pen-gamma;
      If[verbose>1,Print[iterate,": Current penalty= ",pen];];
      
      (* this should already be the case, but to fix round-off errors we do it anyway*)
      (* might be risky: if there is a bug then we are no longer certain that remainder actually is remainder...*)
      remainder2[[maxindices]]=pen*Sign[remainder2[[maxindices]]];
      remainder2[[unweighted]]=0;(*actually 0/0*)
      remainder[[maxindices]]=remainder2[[maxindices]]*weights[[maxindices]];(*so that zeroes are actually zeroes*)
      remainder[[unweighted]]=0;(*or 0.*)
      
      
      (* check if x satisfies fixed point eqn *)
      (* ------------------------------------ *)
      fixedpointtest=Norm[xnew-SoftThreshold[xnew+remainder,pen*weights]];
      If[verbose>1,Print[iterate,": test=",fixedpointtest]];
      
      
      (* append data to datalist *)
      (* ----------------------- *)  
      time=TimeUsed[]-starttime;
      AppendTo[datalist,ReleaseHold[lst]];
      
      
      
            
            
      (* check if candidateindex does not duplicate one that is already in activeset *)
      (* --------------------------------------------------------------------------- *)
      If[Intersection[activeset,candidateindex]!={},
        If[verbose>0,Print[iterate,": Candidate index already in index set. breaking off..."];];
        Break[];
        ];
      
      
      (* check if newindex does not belong to unweighted *)
      (* ----------------------------------------------- *)
      If[Intersection[unweighted,candidateindex]!={},
        If[verbose>0,Print[iterate,": New index in unpenalized set. Breaking off..."];];
        Break[];
        ];
      
           
      (* remove double (origin: gamma=0 somewhere), keep info on index and newindex *)
      (* -------------------------------------------------------------------------- *)
      (* this should never happen*)
      If[gamma==0,
      datalist=Drop[datalist,-1];
      If[verbose>1,Print[iterate,": Duplicate minimizer, removing from list, continuing..."];];
      iterate=iterate-1;
      ];
      
      ];(*end of while loop*)
    
    (* ======================================================== *)
    (* ======================================================== *)
    (* end main loop                                            *)
    (* ======================================================== *)
    (* ======================================================== *)
    
    (* linear interpolation to arrive at exact l1-norm, penalty or discrepancy*)
    (* default case: no interpolation needed*)
    alfaold=0;
    alfanew=1;
    
    (*linear interpolation if target l1 norm is overshot*)
    oldnorm=Norm[xold*weights,1];
    newnorm=Norm[xnew*weights,1];
    If[oldnorm<R<newnorm,
    If[verbose>1,Print["End: Now interpolating for correct final l1-norm."];];
    alfanew=(R-oldnorm)/(newnorm-oldnorm);
    alfaold=(newnorm-R)/(newnorm-oldnorm);
    ];
                    
    (*linear interpolation if Remainder is below target*)
    If[pen<penalty<penold,
    If[verbose>1,Print["End: Now interpolating for correct final penalty."];];
    If[(pen-penalty)/(pen-penold)>alfaold,
    alfaold=(pen-penalty)/(pen-penold);
    alfanew=(penalty-penold)/(pen-penold);
    ];
    ];
    
    (*linear interpolation if discrepancy too low*)
    discr1=Norm[mat.xold-y]^2;
    discr2=Norm[mat.xnew-y]^2;
    If[discr2<discrep<discr1,
    If[verbose>1,Print["End: Now interpolating for correct final discrepancy."];];
    a1=Norm[mat.(xold-xnew)]^2;
    a2=(mat.(xold-xnew)).(mat.xnew-y);
    a3=Norm[mat.xnew-y]^2;
    alf=Select[DeleteCases[Flatten[alf/.Solve[a1 alf^2+2 a2 alf+a3-discrep==0,alf]],_Complex],(0<=#<=1)&];
    If[Length[alf]==1,
    alf=Simplify[alf[[1]]];
    If[alf>alfaold,
    alfaold=alf;
    alfanew=1-alf;
    ];
    ,
    If[verbose>0,Print[iterate,": Couldn't interpolate to match discrepancy..."];];];
    ];
    
    
    (*if interpolation, we need to replace last element of data list*)
    If[alfaold>0,
    datalist=Drop[datalist,-1];
    xnew=alfaold*xold+alfanew*xnew;
    difference=alfaold*differenceold+alfanew*difference;
    remainder=alfaold*remainderold+alfanew*remainder;
    pen=alfaold*penold+alfanew*pen;
    fixedpointtest=Norm[xnew-SoftThreshold[xnew+remainder,pen*weights]];
    (*i don't think we need to update activeset etc*)
    time=TimeUsed[]-starttime;
    AppendTo[datalist,ReleaseHold[lst]];
    ];
    
    If[verbose>1,Print["Final test=",fixedpointtest];];
    

  
      On[Power::"infy"];
      On[Infinity::"indet"];
      
     If[datalist[[1]]===Null,xnew,{datalist,xnew}]
     
     ](*end of module*)








CheckMinimizerList[A_,y_,minlist_,lambdalist_,options___]:=
  Module[{onlyexactnumbers,xfunc,argfunc,res,lam,zero,weights},
  {verbose,weights}={Verbose,Abs[Weights]}/.{options}/.{Verbose->1,Weights->Table[1,{Dimensions[A][[2]]}]};
 
  
  (*check correct sorting order of lambda*)
  sortedok=(lambdalist\[Equal]Sort[lambdalist,Greater]);
  If[Not[sortedok]&&verbose>0,
      Print["List of penalties not in descending order!"];];
  
  (*check for exact numbers only*)
  onlyexactnumbers=(Flatten[Position[{A,y,minlist,lambdalist},_Real]]\[Equal]{});
  If[Not[onlyexactnumbers]&&verbose>0,Print["Only exact numbers allowed!"];];
  
  (*check for lists of equal length*)
  equallength=(Length[minlist]\[Equal]Length[lambdalist]);
  If[Not[equallength]&&verbose>0,Print["Minimizer list and penalty list of different length! Stopping!"];];
    
    (*check for minimumlength=2*)
    morethanone=(Length[minlist]>1&&Length[lambdalist]>1);
    
    If[onlyexactnumbers && sortedok && equallength && morethanone,
      zero=Table[0,{Length[minlist[[1]]]}];
      Do[If[lambdalist[[i+1]]\[Equal]lambdalist[[i]],
          If[verbose>0,Print["Warning, duplicate nodes: ",i-1," and ",i,". Continuing with test."];];
          res=True;(*this allows duplicates*),
          
          (*this is the generic case:*)
        xfunc=Expand[(minlist[[i]](lam-lambdalist[[i+1]])+
                    minlist[[i+1]](lambdalist[[i]]-lam))/(lambdalist[[i]]-
                    lambdalist[[i+1]])];
          argfunc=Expand[xfunc+Transpose[A].(y-A.xfunc)];
          
          res=Simplify[(xfunc-SoftThreshold[argfunc,lam*weights])==zero,
              Assumptions\[RuleDelayed](lambdalist[[i+1]]<lam<lambdalist[[i]])];
          ];
        
        If[Not[res===True]&&verbose>0,
          Print["Problem between nodes: ",i-1," and ",i];
          Break[];];,{i,1,Length[minlist]-1}];
      res,
      (*case of bad input:*)
      
      If[Length[minlist]\[Equal]1&&Length[lambdalist]\[Equal]1,
        If[verbose>0,
          Print["Only one minimizer/penalty in list. Checking this node only."];];
        zero=Table[0,{Length[minlist[[1]]]}];
        (minlist[[1]]-SoftThreshold[minlist[[1]]+Transpose[A].(y-A.minlist[[1]]),
                weights*lambdalist[[1]]])\[Equal]zero,
                Indeterminate]
                ]
        ]




MinimizerInterpolationFunction[minlist_,lambdalist_,var_]:=
  Through[(Interpolation[Transpose[{lambdalist,#}],
              InterpolationOrder\[Rule]1]&/@Transpose[minlist])[var]]
    
    

ThresholdedLandweber[mat_,data_,tau_,options___]:=
  Module[{lst,iterate,starttime,time,x,datalist,remainder,difference,startingpoint,listfunction,
  stoppingcondition,stopcond,weights,tau2},
    
    {startingpoint,listfunction,stoppingcondition,weights}={StartingPoint,HoldForm[ListFunction],
    HoldForm[StoppingCondition],Abs[Weights]}/.{options}/.{StartingPoint->Table[0,{Length[mat[[1]]]}], 
    ListFunction->Null,StoppingCondition->Counter>=1,Weights->Table[1,{Dimensions[mat][[2]]}]};
    
    lst=listfunction/.{DataMisfit->difference, Remainder->remainder, Minimizer->x, Counter->iterate, Time->time};
    stopcond=stoppingcondition/.{DataMisfit->difference, Remainder->remainder, Minimizer->x, Counter->iterate, Time->time};
    
    tau2=tau*weights;
    iterate=0;
    starttime=TimeUsed[];
    x=startingpoint;
    difference=data-mat.x;
    remainder=Transpose[mat].difference;
    time=TimeUsed[]-starttime;
    datalist={ReleaseHold[lst]};
    
    While[Not[ReleaseHold[stopcond]],
      iterate=iterate+1;
      x=SoftThreshold[x+remainder,tau2];
      difference=data-mat.x;
      remainder=Transpose[mat].difference;
      time=TimeUsed[]-starttime;
      AppendTo[datalist,ReleaseHold[lst]];      
        ];
          
    If[datalist[[1]]===Null,x,{datalist,x}]      
  ]


ProjectedLandweber[mat_,data_,radius_,options___]:=
  Module[{lst,iterate,x,datalist,time,difference,remainder,starttime,startingpoint,listfunction,stoppingcondition,stopcond,weights},
  
    {startingpoint,listfunction,stoppingcondition,weights}={StartingPoint,HoldForm[ListFunction],HoldForm[StoppingCondition],
    Abs[Weights]}/.{options}/.{StartingPoint->Table[0,{Length[mat[[1]]]}], 
    ListFunction->Null,StoppingCondition->Counter>=1,Weights->Table[1,{Dimensions[mat][[2]]}]};
    lst=listfunction/.{DataMisfit->difference, Remainder->remainder, Minimizer->x, Counter->iterate, Time->time};
    stopcond=stoppingcondition/.{DataMisfit->difference, Remainder->remainder, Minimizer->x, Counter->iterate, Time->time};
    iterate=0;
    starttime=TimeUsed[];
    x=startingpoint;
    difference=data-mat.x;
    remainder=Transpose[mat].difference;
    time=starttime-TimeUsed[];
    datalist={ReleaseHold[lst]};
        
    While[Not[ReleaseHold[stopcond]],
      iterate=iterate+1;
      x=ProjectionOnL1Ball[x+remainder,radius,Weights->weights];
      difference=data-mat.x;
      remainder=Transpose[mat].difference;
      time=TimeUsed[]-starttime;
      AppendTo[datalist,ReleaseHold[lst]];
      ];
      
   If[datalist[[1]]===Null,x,{datalist,x}] ]


ProjectedSteepestDescent[mat_,data_,radius_,options___]:=
  Module[{startingpoint,listfunction,lst,difference,remainder,time,iterate,starttime,x,num,denom,datalist,stoppingcondition,stopcond},
      
      {startingpoint,listfunction,stoppingcondition,weights}={StartingPoint,HoldForm[ListFunction],
      HoldForm[StoppingCondition],Abs[Weights]}/.{options}/.{StartingPoint->Table[0,{Length[mat[[1]]]}], 
      ListFunction->Null,StoppingCondition->Counter>=1,Weights->Table[1,{Dimensions[mat][[2]]}]};
      
      lst=listfunction/.{DataMisfit->difference, Remainder->remainder, Minimizer->x, Counter->iterate, Time->time};
      stopcond=stoppingcondition/.{DataMisfit->difference, Remainder->remainder, Minimizer->x, Counter->iterate, Time->time};
      
      iterate=0;
      starttime=TimeUsed[];
      x=startingpoint;
      difference=data-mat.x;
      remainder=Transpose[mat].difference;
      num=Norm[remainder]^2;
      denom=Norm[mat.remainder]^2;
      time=starttime-TimeUsed[];
      datalist={ReleaseHold[lst]};
      
    
    While[Not[ReleaseHold[stopcond]],
      iterate=iterate+1;
      x=ProjectionOnL1Ball[x+num/denom*remainder,radius,Weights->weights];
      difference=data-mat.x;
      remainder=Transpose[mat].difference;
      num=Norm[remainder]^2;
      denom=Norm[mat.remainder]^2;
      time=TimeUsed[]-starttime;
      AppendTo[datalist,ReleaseHold[lst]];
      ];
   
     If[datalist[[1]]===Null,x,{datalist,x}] ]
    


AdaptiveLandweber[mat_,data_,radius_,numsteps_,options___]:=
  Module[{listfunction,lst,iterate,x,datalist,rad,deltarad,difference,remainder,starttime,weights},
    
    {listfunction,weights}={HoldForm[ListFunction],Abs[Weights]}/.{options}/.{ListFunction->Null,
    Weights->Table[1,{Dimensions[mat][[2]]}]};
    
    lst=listfunction/.{DataMisfit->difference, Remainder->remainder, Minimizer->x, Counter->iterate, Time->time};
    
    iterate=0;
    starttime=TimeUsed[];
    x=Table[0,{Dimensions[mat][[2]]}];
    rad=0;
    deltarad=radius/numsteps;
    difference=data-mat.x;
    remainder=Transpose[mat].difference;
    time=starttime-TimeUsed[];
    datalist={ReleaseHold[lst]};
    
    
    While[iterate<numsteps,
      iterate=iterate+1;
      rad=rad+deltarad;
      x=ProjectionOnL1Ball[x+remainder,rad,Weights->weights];
      difference=data-mat.x;
      remainder=Transpose[mat].difference;
      time=TimeUsed[]-starttime;
      AppendTo[datalist,ReleaseHold[lst]];
      ];
          
    If[datalist[[1]]===Null,x,{datalist,x}]
    ]


AdaptiveSteepestDescent[mat_,data_,radius_,numsteps_,options___]:=
  Module[{listfunction,lst,iterate,x,datalist,remainder,num,denom,beta,difference,starttime,time,rad,deltarad,weights},
  
    {listfunction,weights}={HoldForm[ListFunction],Abs[Weights]}/.{options}/.{ListFunction->Null,
    Weights->Table[1,{Dimensions[mat][[2]]}]};
    
    lst=listfunction/.{DataMisfit->difference, Remainder->remainder, Minimizer->x, Counter->iterate, Time->time};
    iterate=0;
    starttime=TimeUsed[];
    x=Table[0,{Dimensions[mat][[2]]}];
    rad=0;
    deltarad=radius/numsteps;
    difference=data-mat.x;
    remainder=Transpose[mat].difference;
    num=Norm[remainder]^2;
    denom=Norm[mat.remainder]^2;
    beta=num/denom;
    time=starttime-TimeUsed[];
    datalist={ReleaseHold[lst]};
    
    While[iterate<numsteps,
      iterate=iterate+1;
      rad=rad+deltarad;
      x=ProjectionOnL1Ball[x+beta*remainder,rad,Weights->weights];
      difference=data-mat.x;
      remainder=Transpose[mat].difference;
      num=Norm[remainder]^2;
      denom=Norm[mat.remainder]^2;
      beta=num/denom;
      time=starttime-TimeUsed[];
      AppendTo[datalist,ReleaseHold[lst]];
      
      ];
      
      
    If[datalist[[1]]===Null,x,{datalist,x}]]



End[]         (*end the private context*)

SetAttributes[SoftThreshold,Listable]


Protect[SoftThreshold,ProjectionOnL1Ball,FindMinimizer,CheckMinimizerList,MinimizerInterpolationFunction,ThresholdedLandweber,
ProjectedLandweber,ProjectedSteepestDescent,AdaptiveLandweber,AdaptiveSteepestDescent,DataMisfit, 
Remainder,Minimizer,Counter,Time,ListFunction,StartingPoint,Minimizer,MaximumL1Norm,Verbose,
StoppingCondition,MinimumDiscrepancy,Penalty,Grammian,Support,Tolerance,CandidateIndex,Shift,Gram,Grammian] 


EndPackage[]  (*end the package context*)